
/*
	Compile with -DLED_PD6 to use led on PD6
	Compile with -DLED_PD1 to use led on PD1

	After using led on PD1 you must unbrick because PD1 is the SWD pin
*/

#include <stdint.h>
#include "interrupt.H"
#include "Timer.H"


using namespace std;
using namespace avelino;


class MyTimerListener : public TimerListener {
    public:
        MyTimerListener();
        virtual void timerExpired();
};


#define  RCC_APB2PCENR  *((uint32_t *) 0x40021018)
#define  GPIOD_CFGLR    *((uint32_t *) 0x40011400)
#define  GPIOD_OUTDR    *((uint32_t *) 0x4001140C)
#define  AFIO_PCFR1     *((uint32_t *) 0x40010004)


MyTimerListener::MyTimerListener() {
#if defined LED_PD6
	// enable clock on GPIOD
	RCC_APB2PCENR |= (((uint32_t) 1) << 5);
	// led on PD6
	GPIOD_CFGLR = (GPIOD_CFGLR & 0xF0FFFFFF) | 0x02000000;   // PD6 = output push-pull
#elif defined LED_PD1
	// enable clock on AFIO
	RCC_APB2PCENR |= (((uint32_t) 1) << 0);
	// disable SWD pin on PD1
	AFIO_PCFR1 = (AFIO_PCFR1 & 0xF8FFFFFF) | 0x04000000;
	// enable clock on GPIOD
	RCC_APB2PCENR |= (((uint32_t) 1) << 5);
	// led on PD1
	GPIOD_CFGLR = (GPIOD_CFGLR & 0xFFFFFF0F) | 0x00000020;   // PD1 = output push-pull
#endif
}


void MyTimerListener::timerExpired() {
#if defined LED_PD6
	GPIOD_OUTDR ^= (((uint32_t) 1) << 6);
#elif defined LED_PD1
	GPIOD_OUTDR ^= (((uint32_t) 1) << 1);
#endif
}


int main() {
    interruptInit();
    MyTimerListener myTimerListener;
    Timer::init(myTimerListener, 50_msForTimer);
	while (true)
		asm volatile ("wfi");
}
